import { tupleCV, stringUtf8CV, uintCV, stringAsciiCV } from "micro-stacks/clarity";
import { get_client } from "$stores/client";
import { bytesToHex, ChainID, hexToBytes } from "micro-stacks/common";
import { verify_structured_data_signature } from "./structured-data";
export const domain = {
    name: import.meta.env.VITE_PUBLIC_APP_NAME,
    version: import.meta.env.VITE_PUBLIC_APP_VERSION,
    'chain-id': import.meta.env.VITE_NETWORK === "mainnet" ? ChainID.Mainnet : ChainID.Testnet,
};
export const domainCV = tupleCV({
    name: stringAsciiCV(domain.name),
    version: stringAsciiCV(domain.version),
    'chain-id': uintCV(domain['chain-id']),
});
function signature_data_buffers(data) {
    return {
        signature: hexToBytes(data.signature),
        public_key: hexToBytes(data.publicKey),
    };
}
export function message_date_format(timestamp) {
    // const date = new Date(timestamp);
    // const now = new Date();
    return `${timestamp}`;
}
export function message_to_tuple(message) {
    if (message.subject) // message is Thread
        return tupleCV({
            subject: stringUtf8CV(message.subject),
            body: stringUtf8CV(message.body),
            timestamp: uintCV(message.timestamp)
        });
    return tupleCV({
        body: stringUtf8CV(message.body),
        timestamp: uintCV(message.timestamp),
        thread_id: stringAsciiCV(message._id)
    });
}
export function delete_action_tuple(message_id, is_reply) {
    console.log('delete_action_tuple.message_id: ' + message_id);
    console.log('delete_action_tuple.is_reply: ', is_reply);
    return tupleCV({
        action: stringAsciiCV("delete"),
        message_id: stringAsciiCV(message_id),
        kind: stringAsciiCV(is_reply ? "reply" : "thread")
    });
}
export function verify_signed_message(message) {
    if (!message.signature || !message.author)
        return false;
    const author = typeof message.author === "string" ? hexToBytes(message.author) : message.author;
    const signature = typeof message.signature === "string" ? hexToBytes(message.signature) : message.signature;
    return verify_structured_data_signature(domainCV, message_to_tuple(message), author, signature);
}
export function verify_delete_action(message, signature) {
    if (!message.signature || !message.author)
        return false;
    const author = typeof message.author === "string" ? hexToBytes(message.author) : message.author;
    const sig = typeof signature === "string" ? hexToBytes(signature) : signature;
    const is_reply = typeof message.reply_id !== "undefined";
    return verify_structured_data_signature(domainCV, delete_action_tuple(is_reply ? message.reply_id : message._id, is_reply), author, sig);
}
export async function request_sign_message(message) {
    if (!message.timestamp)
        message.timestamp = Date.now();
    return new Promise(resolve => get_client().signStructuredMessage({
        message: message_to_tuple(message),
        domain,
        onFinish: (result) => resolve(signature_data_buffers(result)),
        onCancel: () => resolve(false)
    }));
}
export async function request_sign_delete_message(message_id, is_reply) {
    return new Promise(resolve => {
        const msg2Sign = delete_action_tuple(message_id, is_reply);
        console.log('request_sign_delete_message.message_id: ' + message_id);
        console.log('request_sign_delete_message.msg2Sign: ', msg2Sign);
        get_client().signStructuredMessage({
            message: msg2Sign,
            domain,
            onFinish: (result) => {
                console.log('request_sign_delete_message.message_id: ' + message_id);
                console.log('request_sign_delete_message.result._id: ' + result._id);
                console.log('request_sign_delete_message.result.signature: ' + result.signature);
                resolve(signature_data_buffers(result));
            },
            onCancel: () => resolve(false)
        });
    });
}
export function is_welformed_signed_message(message) {
    return message.body && message.timestamp && verify_signed_message(message);
}
export function serialise_message(message) {
    const signature = message.signature ? (typeof message.signature !== "string" ? bytesToHex(message.signature) : message.signature) : null;
    const author = message.author ? (typeof message.author !== "string" ? bytesToHex(message.author) : message.author) : null;
    return JSON.stringify({ ...message, signature, author });
}
export function unserialise_message(message_json) {
    try {
        const message = JSON.parse(message_json);
        if (message.signature)
            message.signature = hexToBytes(message.signature);
        if (message.author)
            message.author = hexToBytes(message.author);
        let result = { body: message.body, timestamp: message.timestamp, author: message.author, signature: message.signature };
        if (message.subject) // Thread
            result.subject = message.subject;
        else // Reply
            result._id = message._id;
        return result;
    }
    catch (error) {
        return null;
    }
}
export async function post_message(message) {
    const serialisedMessage = serialise_message(message);
    const host = import.meta.env.VITE_STACKS_VOICE_API_V1;
    const url = message._id ? `/thread/${message._id}/post` : '/thread';
    const result = await fetch(host + url, {
        method: "POST",
        headers: {
            'accept': 'application/json',
            'content-type': 'application/json'
        },
        body: serialisedMessage
    });
    if (!result)
        return false;
    return await result.json();
}
async function delete_message(message_id, reply_id, signature) {
    const path = !reply_id ? `/thread/${message_id}` : `/reply/${message_id}/${reply_id}`;
    const url = import.meta.env.VITE_STACKS_VOICE_API_V1 + path;
    const result = await fetch(url, {
        method: "DELETE",
        headers: {
            'accept': 'application/json',
            'content-type': 'application/json'
        },
        body: JSON.stringify({ signature: bytesToHex(signature) })
    });
    if (!result)
        return false;
    return await result.json();
}
export async function delete_thread(_id, signature) {
    return delete_message(_id, null, signature);
}
export async function delete_reply(thread_id, reply_id, signature) {
    return delete_message(thread_id, reply_id, signature);
}
